// CpuUsage.cpp: implementation of the CCpuUsage class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CpuUsage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCpuUsage::CCpuUsage()
{
    ZeroMemory(&liOldIdleTime, sizeof(liOldIdleTime));
    ZeroMemory(&liOldSystemTime, sizeof(liOldSystemTime));
	//Win32 values
	hKey = NULL;

	m_isWinNT = isWinNT();
	if(!m_isWinNT)
	{
		DWORD dwDataSize;
		DWORD dwType;

		RegOpenKeyEx( HKEY_DYN_DATA,
						   "PerfStats\\StartStat",
						   0,KEY_ALL_ACCESS,
						   &hKey );

		dwDataSize=sizeof(DWORD);
		//DWORD dwCpuUsage = 0;
		RegQueryValueEx( hKey,
						 "KERNEL\\CPUUsage",
						 NULL,&dwType,
						 (LPBYTE)&dwCpuUsage,
						 &dwDataSize );

		RegCloseKey(hKey);
    
		RegOpenKeyEx( HKEY_DYN_DATA,
						   "PerfStats\\StatData",
						   0,KEY_READ,
						   &hKey );
	}
}

CCpuUsage::~CCpuUsage()
{
	if(!m_isWinNT)
	{
		RegCloseKey(hKey);

		if ( RegOpenKeyEx( HKEY_DYN_DATA,
						   "PerfStats\\StopStat",
						   0,KEY_ALL_ACCESS,
						   &hKey ) != ERROR_SUCCESS)
		{
			return;
		}

		DWORD dwDataSize=sizeof(DWORD);
		DWORD dwType;
		DWORD dwCpuUsage;
		RegQueryValueEx( hKey,
						 "KERNEL\\CPUUsage",
						 NULL,&dwType,
						 (LPBYTE)&dwCpuUsage,
						 &dwDataSize );

		RegCloseKey(hKey);
	}
}

BOOL CCpuUsage::isWinNT()
{
	OSVERSIONINFO osv;
	osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if(GetVersionEx( &osv )) {
		switch(osv.dwPlatformId) {
			case VER_PLATFORM_WIN32_NT: // Windows NT
			{
				return TRUE;
			}
			case VER_PLATFORM_WIN32_WINDOWS: // user windows (9x/ME)
			{
				return FALSE;
			}
			default:
			{
				return FALSE;
			}
		}
	} else 
	{
		return FALSE;
	}
}

DWORD CCpuUsage::GetValue()
{
	if(m_isWinNT)
	{
		PROCNTQSI NtQuerySystemInformation;
		SYSTEM_PERFORMANCE_INFORMATION SysPerfInfo;
		SYSTEM_TIME_INFORMATION        SysTimeInfo;
		SYSTEM_BASIC_INFORMATION       SysBaseInfo;
		double                         dbIdleTime = 0;
		double                         dbSystemTime;
		LONG                           status;
	
		NtQuerySystemInformation = (PROCNTQSI)GetProcAddress(
			GetModuleHandle("ntdll.dll"),
			"NtQuerySystemInformation");

		if (!NtQuerySystemInformation)
			return 0;

		// get number of processors in the system
		status = NtQuerySystemInformation(SYS_BASIC_INFO, &SysBaseInfo, sizeof(SysBaseInfo), NULL);
		if (status != NO_ERROR)
			return 0;

		//while(!quiet)
		{
			// get new system time
			status = NtQuerySystemInformation(SYS_TIME_INFO, &SysTimeInfo, sizeof(SysTimeInfo), 0);
			if (status!=NO_ERROR)
				return 0;

			// get new CPU's idle time
			status = NtQuerySystemInformation(SYS_PERFORMANCE_INFO, &SysPerfInfo, sizeof(SysPerfInfo), NULL);
			if (status != NO_ERROR)
				return 0;

			// if it's a first call - skip it
		   if (liOldIdleTime.QuadPart != 0)
		   {
				// CurrentValue = NewValue - OldValue
				dbIdleTime = Li2Double(SysPerfInfo.liIdleTime) - Li2Double(liOldIdleTime);
				dbSystemTime = Li2Double(SysTimeInfo.liKeSystemTime) - Li2Double(liOldSystemTime);

				// CurrentCpuIdle = IdleTime / SystemTime
				dbIdleTime = dbIdleTime / dbSystemTime;

				// CurrentCpuUsage% = 100 - (CurrentCpuIdle * 100) / NumberOfProcessors
				dbIdleTime = 100.0 - dbIdleTime * 100.0 / (double)SysBaseInfo.bKeNumberProcessors + 0.5;

				// put cpu usage in str
				//sprintf(str, "%3d%%", (UINT) dbIdleTime);
				//SetDlgItemText(hMainDlg, IDC_CPUUSG, str);
		   }

			// store new CPU's idle and system time
			liOldIdleTime = SysPerfInfo.liIdleTime;
			liOldSystemTime = SysTimeInfo.liKeSystemTime;
			
			// wait one second and start again
			//Sleep(1000);
		}
		return (DWORD) dbIdleTime;
	} else
	{
		DWORD dwDataSize = sizeof(DWORD);
		//DWORD dwCpuUsage = 50;
		DWORD dwType = 0;
		RegQueryValueEx( hKey,
						 "KERNEL\\CPUUsage",
						 NULL,&dwType,
						 (LPBYTE)&dwCpuUsage,
						 &dwDataSize );
		return dwCpuUsage;
	}
}
