// MemMonBtn.cpp: implementation of the CMemMonBtn class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MemMonBtn.h"
#include "propsdlg.h"
#include <stdio.h>
#include "CpuUsage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMemMonBtn::CMemMonBtn()
{
	m_showCPU = TRUE;
	m_cpu = NULL;
	ZeroMemory(m_values, sizeof(m_values));
	lastValid = -1;

	m_version = 0;
	m_ObjRefCount = 1;
	m_container = NULL;

	hTimerThread = NULL;

	hStop = CreateEvent(NULL, FALSE, FALSE, NULL);
	StartTimer();

	m_data[0].show = TRUE;
	m_data[0].color = RGB(0xFF, 0, 0);
	m_data[1].show = TRUE;
	m_data[1].color = RGB(0xFF, 0xFF, 0);
	m_data[2].show = TRUE;
	m_data[2].color = RGB(0, 0xFF, 0);

	g_DllRefCount++;
}

CMemMonBtn::~CMemMonBtn()
{
	SetEvent(hStop);
	WaitForSingleObject(hTimerThread, INFINITE);
	CloseHandle(hTimerThread);
	CloseHandle(hStop);

	g_DllRefCount--;
}


//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CMemMonBtn::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	   {
	   *ppReturn = (ITlbButton*) this;
	   }

	if(IsEqualIID(riid, IID_IUnknown))
	   {
	   *ppReturn = (IUnknown*) this;
	   }

	if(*ppReturn)
	   {
	   (*(LPUNKNOWN*)ppReturn)->AddRef();
	   return S_OK;
	   }

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CMemMonBtn::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CMemMonBtn::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}


//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////


// Load parameters for button
STDMETHODIMP CMemMonBtn::Load(IStream *data)
{
	DWORD loadedVersion = 0;
	ULONG cbRead = 0;
	data->Read(&loadedVersion, sizeof(DWORD), &cbRead);
	data->Read(m_data, sizeof(m_data), &cbRead);
	data->Read(&m_showCPU, sizeof(m_showCPU), &cbRead);
	return S_OK;
}

// Save button parameters
STDMETHODIMP CMemMonBtn::Save(IStream *data)
{
	ULONG cbWritten;
	data->Write(&m_version, sizeof(DWORD), &cbWritten);
	data->Write(m_data, sizeof(m_data), &cbWritten);
	data->Write(&m_showCPU, sizeof(m_showCPU), &cbWritten);
	return S_OK;
}

// Sets button rectangle
STDMETHODIMP CMemMonBtn::SetRect(RECT *rcItem)
{
	m_rc = rcItem[0];
	return S_OK;
}



// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CMemMonBtn::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	return S_OK;
}

STDMETHODIMP CMemMonBtn::SetID(int btnID)
{
	m_ID = btnID;
	return S_OK;
}

STDMETHODIMP CMemMonBtn::Draw(HANDLE hDC, RECT *rcItem)
{
	HDC dc = (HDC) hDC;

	RECT rc = *rcItem;
	rc.left++;
	rc.right--;
	rc.bottom--;
	rc.top++;

	HBRUSH hBlackBrush = CreateSolidBrush(0);
	FillRect(dc, rcItem, hBlackBrush);
	DeleteObject(hBlackBrush);

	if(m_showCPU)
	{
		rc.left += 5;
	}
	
	HPEN pen[3];
	for(int i=0; i < 3; i++)
	{
		pen[i] = CreatePen(PS_SOLID, 1, m_data[i].color);
	}
	HPEN oldPen = (HPEN) SelectObject(dc, pen[0]);

	if(lastValid >= 0)
	{
		POINT oldPT[3];
		for(int p=0; p < 3; p++)
		{
			if(!m_data[p].show) continue;
			CalcPoint(oldPT[p], &rc, 0, p);
			for(int i=1; i <= lastValid; i++)
			{
				POINT pt;
				CalcPoint(pt, &rc, i, p);
				if(oldPT[p].x != pt.x || oldPT[p].y != pt.y)
				{
					SelectObject(dc, pen[p]);
					MoveToEx(dc, oldPT[p].x, oldPT[p].y, NULL);
					LineTo(dc, pt.x, pt.y);
				}
				oldPT[p] = pt;
			}
			MoveToEx(dc, oldPT[p].x, oldPT[p].y, NULL);
			LineTo(dc, oldPT[p].x, oldPT[p].y);
		}
	}
	SelectObject(dc, oldPen);
	for(i=0; i < 3; i++)
	{
		DeleteObject(pen[i]);
	}

	if(m_showCPU)
	{
		rc.left -= 5;
		rc.right = rc.left + 5;

		HBRUSH brCPU = CreateSolidBrush(RGB(0, 0xFF, 0));

		rc.top = rc.bottom - (((double)rc.bottom - (double)rc.top) / 100.0) * ((double)m_cpu);
		FillRect(dc, &rc, brCPU);

		DeleteObject(brCPU);
	}

	return S_OK;
}

STDMETHODIMP CMemMonBtn::OnMouseOver(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CMemMonBtn::OnMouseLeave(long keys)
{
	return S_OK;
}

STDMETHODIMP CMemMonBtn::OnMouseEnter(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CMemMonBtn::Create(IStream* data)
{
	if(m_container)
	{
		HWND hWndBand = m_container->GetBandWND();
		CPropsDlg dlg;
		memcpy(dlg.m_data, m_data, sizeof(m_data));
		dlg.m_showCPU = m_showCPU;
		if(dlg.DoModal(hWndBand) == IDOK)
		{
			m_showCPU = dlg.m_showCPU;
			memcpy(m_data, dlg.m_data, sizeof(m_data));
			if(m_container)
			{
				m_container->UpdateButtonInfo(UPDATE_REDRAW);
			}
			return Save(data);
		}
	}
	return S_FALSE;
}


STDMETHODIMP CMemMonBtn::StateChanged(int state, int* needRedraw)
{
	needRedraw[0] = TRUE;
	return S_OK;
}

STDMETHODIMP CMemMonBtn::GetDescription(char* szDescription)
{
	if(szDescription)
	{
		SYSTEMTIME tm;
		GetLocalTime(&tm);
		GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, &tm, NULL, szDescription, 255);
		return S_OK;
		//strcpy(szDescription, "Clock");
	}
	return S_FALSE;
}

STDMETHODIMP CMemMonBtn::OnLButtonDown(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CMemMonBtn::OnLButtonUp(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CMemMonBtn::Run()
{
	return S_OK;
}



STDMETHODIMP CMemMonBtn::DoAction(int action)
{
	if(action == CTMS_PROPERTIES)
	{
		if(m_container)
		{
			HWND hWndBand = m_container->GetBandWND();
			CPropsDlg dlg;
			memcpy(dlg.m_data, m_data, sizeof(m_data));
			dlg.m_showCPU = m_showCPU;
			if(dlg.DoModal(hWndBand) == IDOK)
			{
				m_showCPU = dlg.m_showCPU;
				memcpy(m_data, dlg.m_data, sizeof(m_data));
				if(m_container)
				{
					m_container->UpdateButtonInfo(UPDATE_REDRAW);
				}
				return m_container->SaveButton();
			}
		}
	}
	return S_OK;
}

STDMETHODIMP_(int) CMemMonBtn::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	return 0;
}

STDMETHODIMP CMemMonBtn::InvokeCommand(int ID)
{
	return S_OK;
}


STDMETHODIMP CMemMonBtn::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	return S_OK;
}

STDMETHODIMP CMemMonBtn::GetIconLocation(LPSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CMemMonBtn::SetIconLocation(LPCSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CMemMonBtn::SetName(LPCSTR szName)
{
	//strcpy(m_Name, szName);
	return S_OK;
}

STDMETHODIMP CMemMonBtn::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = CTMS_PROPERTIES;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE ||
	   btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		//int n = m_showSeconds ? 6 : 4;
		//int nn = m_showSeconds ? 2 : 1;
		//btninfo->szBtnSize.cx = (m_digWidth + 1) * n + 6 + (m_dotWidth + 1) * nn;
	}
	if(btninfo->dwMask & BTNINFO_MASK_DESCRIPTION)
	{
		char* szDescription = (char*) btninfo->strData;
		MEMORYSTATUS mem;
		GlobalMemoryStatus(&mem);
		DWORD dwTotalPhys = mem.dwTotalPhys / 1024;
		DWORD dwAvailPhys = mem.dwAvailPhys / 1024;
		DWORD dwTotalVirtual = mem.dwTotalVirtual / 1024;
		DWORD dwAvailVirtual = mem.dwAvailVirtual / 1024;
		DWORD dwTotalPageFile = mem.dwTotalPageFile / 1024;
		DWORD dwAvailPageFile = mem.dwAvailPageFile / 1024;
		sprintf(szDescription, "Virtual Memory:\t%dK / %dK\nPhysical memory:\t%dK / %dK\nTotal Memory usage:\t%dK / %dK", 
			dwTotalVirtual-dwAvailVirtual, dwTotalVirtual, 
			dwTotalPhys-dwAvailPhys, dwTotalPhys, dwTotalPageFile-dwAvailPageFile, dwTotalPageFile);
	}
	return S_OK;
}

DWORD WINAPI CMemMonBtn::TimerTrdProc(LPVOID lpParam)
{
	CoInitialize(NULL);
	CMemMonBtn* tr = (CMemMonBtn*) lpParam;
	tr->TimerThread();
	CoUninitialize();
	return 0;
}

void CMemMonBtn::TimerThread()
{
	int ii = 0;
	CCpuUsage cpu;
	while(true)
	{
		DWORD res = WaitForSingleObject(hStop, 500);
		if(res != WAIT_TIMEOUT)
		{
			break;
		}
		m_cpu = cpu.GetValue();
		//if(!ii)
		//{
			AddValue();
		//}
		if(m_container && m_ID)
		{
			m_container->UpdateButtonInfo(UPDATE_REDRAW);
		}
		//if(ii) ii = 0;
		//else ii = 1;
	}
}

void CMemMonBtn::StartTimer()
{
	DWORD id;	
	hTimerThread = CreateThread(NULL, 0, CMemMonBtn::TimerTrdProc, (LPVOID) this, 0, &id);
}


void CMemMonBtn::AddValue()
{
	MEMORYSTATUS mem;
	GlobalMemoryStatus(&mem);
	DWORD dwTotalPhys = mem.dwTotalPhys / 1024;
	DWORD dwAvailPhys = mem.dwAvailPhys / 1024;
	DWORD dwTotalVirtual = mem.dwTotalVirtual / 1024;
	DWORD dwAvailVirtual = mem.dwAvailVirtual / 1024;
	DWORD dwTotalPageFile = mem.dwTotalPageFile / 1024;
	DWORD dwAvailPageFile = mem.dwAvailPageFile / 1024;

	if(lastValid < 49)
	{
		lastValid++;
	}
	for(int i=lastValid; i > 0; i--)
	{
		m_values[i] = m_values[i-1];
	}

	m_values[0].pageUsed = dwTotalPageFile-dwAvailPageFile;
	m_values[0].pageTotal = dwTotalPageFile;

	m_values[0].physUsed = dwTotalPhys-dwAvailPhys;
	m_values[0].physTotal = dwTotalPhys;

	m_values[0].virtUsed = dwTotalVirtual-dwAvailVirtual;
	m_values[0].virtTotal = dwTotalVirtual;
}

void CMemMonBtn::CalcPoint(POINT &pt, LPRECT rc, int index, int type)
{
	DWORD used;
	DWORD total;
	switch(type)
	{
	case 0:
		total = m_values[index].physTotal;
		used = m_values[index].physUsed;
		break;
	case 1:
		total = m_values[index].virtTotal;
		used = m_values[index].virtUsed;
		break;
	case 2:
		total = m_values[index].pageTotal;
		used = m_values[index].pageUsed;
		break;
	}
	int rcWidth = rc->right - rc->left;
	pt.x = rc->right - ((double)rcWidth / 50.0) * index;

	int rcHeight = rc->bottom - rc->top;
	pt.y = rc->bottom - ((double)rcHeight / (double)total) * used;
}

STDMETHODIMP CMemMonBtn::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	return S_FALSE;
}

