// ClockButton.cpp: implementation of the CClockButton class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ClockButton.h"
#include "resource.h"
#include "utils.h"
#include "PropsDlg.h"
#include <shellapi.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CClockButton::CClockButton()
{
	m_showSeconds = FALSE;
	m_blinkDots = TRUE;

	m_state = 0;
	m_ID = 0;
	m_version = 1;
	m_container = NULL;

	m_bmpDigits = NULL;
	m_digWidth = 0;
	m_digHeight = 0;
	m_digLoaded = FALSE;

	m_bmpDots = NULL;
	m_dotWidth = NULL;
	m_dotHeight = NULL;

	m_ObjRefCount = 1;
	g_DllRefCount++;

	hTimerThread = NULL;
	tik = 0;

	hStop = CreateEvent(NULL, FALSE, FALSE, NULL);
	StartTimer();
}

CClockButton::~CClockButton()
{
	SetEvent(hStop);
	WaitForSingleObject(hTimerThread, INFINITE);
	CloseHandle(hTimerThread);
	CloseHandle(hStop);

	if(m_bmpDigits)
	{
		DeleteObject(m_bmpDigits);
	}

	g_DllRefCount--;
}


//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CClockButton::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	   {
	   *ppReturn = (IDataObject*) this;
	   }

	if(IsEqualIID(riid, IID_IUnknown))
	   {
	   *ppReturn = (IUnknown*) this;
	   }

	if(*ppReturn)
	   {
	   (*(LPUNKNOWN*)ppReturn)->AddRef();
	   return S_OK;
	   }

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CClockButton::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CClockButton::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}

//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////


// Load parameters for button
STDMETHODIMP CClockButton::Load(IStream *data)
{
	DWORD loadedVersion = 0;
	ULONG cbRead = 0;
	data->Read(&loadedVersion, sizeof(DWORD), &cbRead);
	data->Read(&m_showSeconds, sizeof(BOOL), &cbRead);
	data->Read(&m_blinkDots, sizeof(BOOL), &cbRead);
	return S_OK;
}

// Save button parameters
STDMETHODIMP CClockButton::Save(IStream *data)
{
	ULONG cbWritten;
	data->Write(&m_version, sizeof(DWORD), &cbWritten);
	data->Write(&m_showSeconds, sizeof(BOOL), &cbWritten);
	data->Write(&m_blinkDots, sizeof(BOOL), &cbWritten);
	return S_OK;
}

// Sets button rectangle
STDMETHODIMP CClockButton::SetRect(RECT *rcItem)
{
	m_rc = rcItem[0];
	return S_OK;
}



// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CClockButton::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	LoadDigits();
	return S_OK;
}

STDMETHODIMP CClockButton::SetID(int btnID)
{
	m_ID = btnID;
	return S_OK;
}

STDMETHODIMP CClockButton::Draw(HANDLE hDC, RECT *rcItem)
{
	HDC dc = (HDC) hDC;

	if(!m_container) return S_FALSE;

	CONTAINER_PARAMS params;
	params.cbSize = sizeof(CONTAINER_PARAMS);
	m_container->GetParams(&params);

	int x = rcItem->left + 3;
	int y = rcItem->top + (rcItem->bottom - rcItem->top) / 2 - m_digHeight / 2;

	if(params.style & CTS_ISMENU)
	{
		RECT rcMargins;
		COLORREF color;
		m_container->GetTextProps(&rcMargins, &color);
		x = rcItem->left + rcMargins.left;
	}

	if(!m_digLoaded)
	{
		LoadDigits();
	}
	
	HDC hdcSrc = CreateCompatibleDC(dc);
	HDC hdcDst = CreateCompatibleDC(dc);
	
	HBITMAP hbmpBtn = CreateCompatibleBitmap(dc, m_digWidth, m_digHeight);

	HBITMAP oldBitmap1 = (HBITMAP) SelectObject(hdcSrc, m_bmpDigits);

	int digs[6];

	SYSTEMTIME tm;
	GetLocalTime(&tm);
	char str[10];
	itoa(tm.wHour, str, 10);
	if(!str[1])
	{
		digs[0] = -1;
		digs[1] = str[0] - '0';
	} else
	{
		digs[0] = str[0] - '0';
		digs[1] = str[1] - '0';
	}

	itoa(tm.wMinute, str, 10);
	if(str[1])
	{
		digs[2] = str[0] - '0';
		digs[3] = str[1] - '0';
	} else
	{
		digs[2] = 0;
		digs[3] = str[0] - '0';
	}

	itoa(tm.wSecond, str, 10);
	if(str[1])
	{
		digs[4] = str[0] - '0';
		digs[5] = str[1] - '0';
	} else
	{
		digs[4] = 0;
		digs[5] = str[0] - '0';
	}

	int dotsTop = y + m_digHeight / 2 - m_dotHeight / 2;

	int n = m_showSeconds ? 6 : 4;

	for(int i=0; i < n; i++)
	{
		if(i == 2 || i == 4)
		{
			if(tik || !m_blinkDots)
			{
				DrawTransBitmap(dc, m_bmpDots, x, dotsTop, m_dotWidth, m_dotHeight, RGB(0xFF, 0, 0xFF));
			}
			x += m_dotWidth + 1;
		}
		if(digs[i] >= 0)
		{
			HBITMAP oldBitmap2 = (HBITMAP) SelectObject(hdcDst, hbmpBtn);
			BitBlt(hdcDst, 0, 0, m_digWidth, m_digHeight, hdcSrc, digs[i] * m_digWidth, 0, SRCCOPY);
			SelectObject(hdcDst, oldBitmap2);
			DrawTransBitmap(dc, hbmpBtn, x, y, m_digWidth, m_digHeight, RGB(0xFF, 0, 0xFF));
		}
		x += m_digWidth + 1;
	}
	
	SelectObject(hdcSrc, oldBitmap1);
	DeleteDC(hdcSrc);
	DeleteDC(hdcDst);
	DeleteObject(hbmpBtn);

	return S_OK;
}

STDMETHODIMP CClockButton::OnMouseOver(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CClockButton::OnMouseLeave(long keys)
{
	return S_OK;
}

STDMETHODIMP CClockButton::OnMouseEnter(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CClockButton::Create(IStream* data)
{
	if(m_container)
	{
		HWND hWndBand = m_container->GetBandWND();
		CPropsDlg dlg;
		dlg.m_blinkDots = m_blinkDots;
		dlg.m_showSeconds = m_showSeconds;
		if(dlg.DoModal(hWndBand) == IDOK)
		{
			m_blinkDots = dlg.m_blinkDots;
			m_showSeconds = dlg.m_showSeconds;
			return Save(data);
		}
	}
	return S_FALSE;
}


STDMETHODIMP CClockButton::StateChanged(int state, int* needRedraw)
{
	needRedraw[0] = TRUE;
	m_state = state;
	return S_OK;
}

STDMETHODIMP CClockButton::GetDescription(char* szDescription)
{
	if(szDescription)
	{
		SYSTEMTIME tm;
		GetLocalTime(&tm);
		GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, &tm, NULL, szDescription, 255);
		return S_OK;
		//strcpy(szDescription, "Clock");
	}
	return S_FALSE;
}

STDMETHODIMP CClockButton::OnLButtonDown(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CClockButton::OnLButtonUp(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CClockButton::Run()
{
	ShellExecute(NULL, NULL, "rundll32.exe", "shell32.dll,Control_RunDLL timedate.cpl", NULL, SW_NORMAL);
	return S_OK;
}



STDMETHODIMP CClockButton::DoAction(int action)
{
	if(action == CTMS_PROPERTIES)
	{
		if(m_container)
		{
			HWND hWndBand = m_container->GetBandWND();
			CPropsDlg dlg;
			dlg.m_blinkDots = m_blinkDots;
			dlg.m_showSeconds = m_showSeconds;
			if(dlg.DoModal(hWndBand) == IDOK)
			{
				BOOL OldshowSeconds = m_showSeconds;
				m_blinkDots = dlg.m_blinkDots;
				m_showSeconds = dlg.m_showSeconds;
				if(OldshowSeconds != m_showSeconds)
				{
					m_container->UpdateButtonInfo(UPDATE_REDRAW | UPDATE_SIZE);
				}
				return m_container->SaveButton();
			}
		}
	}
	return S_OK;
}

STDMETHODIMP_(int) CClockButton::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	return 0;
}

STDMETHODIMP CClockButton::InvokeCommand(int ID)
{
	return S_OK;
}


STDMETHODIMP CClockButton::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	return S_OK;
}

STDMETHODIMP CClockButton::GetIconLocation(LPSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CClockButton::SetIconLocation(LPCSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CClockButton::SetName(LPCSTR szName)
{
	strcpy(m_Name, szName);
	return S_OK;
}

void CClockButton::LoadDigits()
{
	if(m_bmpDigits)
	{
		DeleteObject(m_bmpDigits);
	}
	m_bmpDigits = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_DIGITS));
	BITMAP bmpInfo;
	GetObject(m_bmpDigits, sizeof(bmpInfo), &bmpInfo);
	m_digWidth = bmpInfo.bmWidth / 10;
	m_digHeight = bmpInfo.bmHeight;
	
	m_bmpDots = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_DOTS));
	GetObject(m_bmpDots, sizeof(bmpInfo), &bmpInfo);
	m_dotWidth = bmpInfo.bmWidth;
	m_dotHeight = bmpInfo.bmHeight;

	m_digLoaded = TRUE;
}


DWORD WINAPI CClockButton::TimerTrdProc(LPVOID lpParam)
{
	CoInitialize(NULL);
	CClockButton* tr = (CClockButton*) lpParam;
	//SetThreadPriority(tr->m_hThread, THREAD_PRIORITY_IDLE);
	tr->TimerThread();
	CoUninitialize();
	return 0;
}

void CClockButton::TimerThread()
{
	while(true)
	{
		DWORD res = WaitForSingleObject(hStop, 500);
		if(res == WAIT_OBJECT_0)
		{
			break;
		}
		tik = tik ? 0 : 1;
		if(m_container && m_ID)
		{
			m_container->UpdateButtonInfo(UPDATE_REDRAW);
		}
	}
}

void CClockButton::StartTimer()
{
	DWORD id;	
	hTimerThread = CreateThread(NULL, 0, CClockButton::TimerTrdProc, (LPVOID) this, 0, &id);
}


STDMETHODIMP CClockButton::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = CTMS_PROPERTIES;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE ||
	   btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		int n = m_showSeconds ? 6 : 4;
		int nn = m_showSeconds ? 2 : 1;
		btninfo->szBtnSize.cx = (m_digWidth + 1) * n + 6 + (m_dotWidth + 1) * nn;
	}
	if(btninfo->dwMask & BTNINFO_MASK_DESCRIPTION)
	{
		char* szDescription = (char*) btninfo->strData;
		SYSTEMTIME tm;
		GetLocalTime(&tm);
		GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, &tm, NULL, szDescription, sizeof(btninfo->strData));
	}
	return S_OK;
}

STDMETHODIMP CClockButton::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	return S_OK;
}

