// Mixer.cpp: implementation of the CMixer class.
//
//////////////////////////////////////////////////////////////////////

#include "globals.h"
#include "Mixer.h"


struct ICONS2TYPE
{
	DWORD type;
	int iconIndex;
};


ICONS2TYPE iconsIndexes[] = 
{
	{MIXERLINE_COMPONENTTYPE_DST_UNDEFINED,		0},
	{MIXERLINE_COMPONENTTYPE_DST_DIGITAL,		5},
	{MIXERLINE_COMPONENTTYPE_DST_LINE,			0},
	{MIXERLINE_COMPONENTTYPE_DST_MONITOR,		0},
	{MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,		0},
	{MIXERLINE_COMPONENTTYPE_DST_HEADPHONES,	0},
	{MIXERLINE_COMPONENTTYPE_DST_TELEPHONE,		0},
	{MIXERLINE_COMPONENTTYPE_DST_WAVEIN,		0},
	{MIXERLINE_COMPONENTTYPE_DST_VOICEIN,		0},
	{MIXERLINE_COMPONENTTYPE_SRC_UNDEFINED,		0},
	{MIXERLINE_COMPONENTTYPE_SRC_DIGITAL,		5},
	{MIXERLINE_COMPONENTTYPE_SRC_LINE,			0},
	{MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE,	2},
	{MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER,	3},
	{MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC,	1},
	{MIXERLINE_COMPONENTTYPE_SRC_TELEPHONE,		4},
	{MIXERLINE_COMPONENTTYPE_SRC_PCSPEAKER,		0},
	{MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT,		0},
	{MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY,		0},
	{MIXERLINE_COMPONENTTYPE_SRC_ANALOG,		0}
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMixer::CMixer(HWND hWndNotify, DWORD dwComponentType)
{
	g_DllRefCount++;

	m_controlName[0] = 0;
	m_typeName[0] = 0;

	isValidVolume = FALSE;
	isValidMute = FALSE;
	MMRESULT rc;
	rc = mixerOpen(&hMixer, 0, (ULONG) hWndNotify, 0, CALLBACK_WINDOW);
	if(rc == MMSYSERR_NOERROR)
	{
		MIXERLINE mxl;
		ZeroMemory(&mxl,sizeof(mxl));
		mxl.cbStruct = sizeof(mxl);
		
		if(HIWORD(dwComponentType) == 0xFFFF)
		{
			mxl.dwDestination = LOWORD(dwComponentType);
			rc = mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl,
							   MIXER_GETLINEINFOF_DESTINATION);
		} else
		{
			mxl.dwDestination = LOWORD(dwComponentType);
			mxl.dwSource = HIWORD(dwComponentType);
			rc = mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl,
							   MIXER_GETLINEINFOF_SOURCE);
		}
		strcpy(m_controlName, mxl.szName);

		//find type name
		{
			MIXERLINE mxl;
			mxl.cbStruct      = sizeof(mxl);
			mxl.dwDestination = LOWORD(dwComponentType);
			rc = mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_DESTINATION);
			strcpy(m_typeName, mxl.szName);
		}
		
		dwLineID = mxl.dwLineID;
		if(rc == MMSYSERR_NOERROR)
		{
			m_iconIndex = -1;
			for(int i=0; i < sizeof(iconsIndexes) / sizeof(iconsIndexes[0]); i++)
			{
				if(iconsIndexes[i].type == mxl.dwComponentType)
				{
					m_iconIndex = iconsIndexes[i].iconIndex;
					break;
				}
			}
			MIXERLINECONTROLS mxlc;
			MIXERCONTROL mxc;
			MIXERCONTROL mxcMute;
		
			ZeroMemory(&mxlc, sizeof(mxlc));
			mxlc.cbStruct = sizeof(mxlc);
			mxlc.dwLineID = dwLineID;
			mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
			mxlc.cControls = 1;
			mxlc.cbmxctrl = sizeof(mxcMute);
			mxlc.pamxctrl = &mxcMute;
			ZeroMemory(&mxc, sizeof(mxcMute));
			mxcMute.cbStruct = sizeof(mxcMute);
			rc = mixerGetLineControls((HMIXEROBJ)hMixer,&mxlc,
									   MIXER_GETLINECONTROLSF_ONEBYTYPE);

			dwMuteCtlID = mxcMute.dwControlID;

			ZeroMemory(&mxlc, sizeof(mxlc));
			mxlc.cbStruct = sizeof(mxlc);
			mxlc.dwLineID = dwLineID;
			mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
			mxlc.cControls = 1;
			mxlc.cbmxctrl = sizeof(mxc);
			mxlc.pamxctrl = &mxc;
			ZeroMemory(&mxc, sizeof(mxc));
			mxc.cbStruct = sizeof(mxc);
			rc = mixerGetLineControls((HMIXEROBJ)hMixer,&mxlc,
									   MIXER_GETLINECONTROLSF_ONEBYTYPE);
			dwVolumeCtlID = mxc.dwControlID;
			minVol = mxc.Bounds.lMinimum;
			maxVol = mxc.Bounds.lMaximum;
		
		}
	}
}

CMixer::~CMixer()
{
	g_DllRefCount--;
	if(hMixer)
	{
		mixerClose(hMixer);
	}
}

int CMixer::GetVolume()
{
	if(isValidVolume)
	{
		return volume;
	}
	isValidVolume = TRUE;

	MIXERCONTROLDETAILS mxcd;             // Gets the control values.
	MIXERCONTROLDETAILS_SIGNED volStruct; // Gets the control values.

	// Initialize the MIXERCONTROLDETAILS structure
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct = sizeof(mxcd);
	mxcd.cbDetails = sizeof(volStruct);
	mxcd.dwControlID = dwVolumeCtlID;
	mxcd.paDetails = &volStruct;
	mxcd.cChannels = 1;

	// Get the current value of the peakmeter control. Typically, you
	// would set a timer in your program to query the volume every 10th
	// of a second or so.
	MMRESULT rc = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
								 MIXER_GETCONTROLDETAILSF_VALUE);
	if (MMSYSERR_NOERROR == rc) 
	{
		volume = volStruct.lValue;

		// Get the absolute value of the volume.
		if (volume < 0)
			volume = -volume;
	}
	return volume;
}

void CMixer::SetVolume(int volume)
{
	if(isValidVolume && this->volume == volume)
	{
		return;
	}
	isValidVolume = TRUE;
	this->volume = volume;
	MIXERCONTROLDETAILS mxcd;             // Gets the control values.
	MIXERCONTROLDETAILS_SIGNED volStruct; // Gets the control values.
	//long volume;                          // Holds the final volume value.

	// Initialize the MIXERCONTROLDETAILS structure
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct = sizeof(mxcd);
	mxcd.cbDetails = sizeof(volStruct);
	mxcd.dwControlID = dwVolumeCtlID;
	mxcd.paDetails = &volStruct;
	mxcd.cChannels = 1;

	// Get the current value of the peakmeter control. Typically, you
	// would set a timer in your program to query the volume every 10th
	// of a second or so.
	volStruct.lValue = volume;
	MMRESULT rc = mixerSetControlDetails((HMIXEROBJ)hMixer, &mxcd,
								 MIXER_GETCONTROLDETAILSF_VALUE);
}

BOOL CMixer::isMute()
{
	if(isValidMute)
	{
		return mute;
	}
	isValidMute = TRUE;
	MIXERCONTROLDETAILS mxcd;             // Gets the control values.
	MIXERCONTROLDETAILS_BOOLEAN muteStruct; // Gets the control values.
	//long volume;                          // Holds the final volume value.

	// Initialize the MIXERCONTROLDETAILS structure
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct = sizeof(mxcd);
	mxcd.cbDetails = sizeof(muteStruct);
	mxcd.dwControlID = dwMuteCtlID;
	mxcd.paDetails = &muteStruct;
	mxcd.cChannels = 1;

	// Get the current value of the peakmeter control. Typically, you
	// would set a timer in your program to query the volume every 10th
	// of a second or so.
	MMRESULT rc = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
								 MIXER_GETCONTROLDETAILSF_VALUE);
	if (MMSYSERR_NOERROR == rc) 
	{
		mute = muteStruct.fValue;
	} else 
	{
		mute = FALSE;
	}
	return mute;
}

void CMixer::SetMute(BOOL fMute)
{
	mute = fMute;
	isValidMute = TRUE;
	MIXERCONTROLDETAILS mxcd;             // Gets the control values.
	MIXERCONTROLDETAILS_BOOLEAN muteStruct; // Gets the control values.
	//long volume;                          // Holds the final volume value.

	// Initialize the MIXERCONTROLDETAILS structure
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct = sizeof(mxcd);
	mxcd.cbDetails = sizeof(muteStruct);
	mxcd.dwControlID = dwMuteCtlID;
	mxcd.paDetails = &muteStruct;
	mxcd.cChannels = 1;

	muteStruct.fValue = fMute;

	MMRESULT rc = mixerSetControlDetails((HMIXEROBJ)hMixer, &mxcd,
								 MIXER_GETCONTROLDETAILSF_VALUE);
}

BOOL CMixer::isThis(HMIXER hMix, DWORD ctlID)
{
	if(hMix == hMixer)
	{
		if(ctlID == dwVolumeCtlID)
		{
			isValidVolume = FALSE;
			return TRUE;
		}
		if(ctlID == dwMuteCtlID)
		{
			isValidMute = FALSE;
			return TRUE;
		}
	}
	return FALSE;
}
