// PropsDlg.cpp: implementation of the CPropsDlg class.
//
//////////////////////////////////////////////////////////////////////

#include "globals.h"
#include "PropsDlg.h"
#include <commctrl.h>
#include <Shlwapi.h>
#include "Tracker.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPropsDlg::CPropsDlg() : CTrueDialog(IDD_PROPS)
{
	m_hRes = NULL;
	m_isVertical = FALSE;
	trk = new CTracker();
	m_items = 0;
	count = 0;
	m_skin[0] = 0;
}

CPropsDlg::~CPropsDlg()
{
	if(m_items) free(m_items);
	delete trk;
}

void CPropsDlg::OnInitDialog()
{
	char str[255];
	LoadString(m_hRes, IDS_CTLNAME, str, 255);
	HWND hLst = GetDlgItem(m_hWnd, IDC_LSTCONTROLS);
	LVCOLUMN lvc;
	lvc.mask = LVCF_WIDTH | LVCF_TEXT;
	lvc.iSubItem = 0;
	lvc.cx = 200;
	lvc.pszText = str;
	ListView_InsertColumn(hLst, 0, &lvc);
	
	LoadString(m_hRes, IDS_TYPE, str, 255);
	lvc.mask = LVCF_WIDTH | LVCF_TEXT;
	lvc.iSubItem = 0;
	lvc.cx = 100;
	lvc.pszText = str;
	ListView_InsertColumn(hLst, 1, &lvc);

	ListView_SetExtendedListViewStyle(hLst, LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);

	SendDlgItemMessage(m_hWnd, IDC_ISVERTICAL, BM_SETCHECK, m_isVertical, 0);
	SendDlgItemMessage(m_hWnd, IDC_SHOWTYPE, BM_SETCHECK, m_showType, 0);

	LoadLists();
	FillSkins();
	
	SetSkin();
}

void CPropsDlg::LoadLists()
{
	HMIXER hMixer;
	MMRESULT rc;
	rc = mixerOpen(&hMixer, 0, 0, 0, 0);
	if(rc != MMSYSERR_NOERROR) return;
	MIXERCAPS mxcaps;
    rc = mixerGetDevCaps((UINT)hMixer, &mxcaps, sizeof(mxcaps));
    if (MMSYSERR_NOERROR == rc)
	{
		for (DWORD u = 0; u < mxcaps.cDestinations; u++)
		{
			MIXERLINE mxl;
			mxl.cbStruct      = sizeof(mxl);
			mxl.dwDestination = u;
			rc = mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_DESTINATION);
			if(MMSYSERR_NOERROR == rc)
			{
				if(mxl.cControls)
				{
					MIXERLINECONTROLS mxlc;
					MIXERCONTROL mxc;

					ZeroMemory(&mxlc, sizeof(mxlc));
					mxlc.cbStruct = sizeof(mxlc);
					mxlc.dwLineID = mxl.dwLineID;
					mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
					mxlc.cControls = 1;
					mxlc.cbmxctrl = sizeof(mxc);
					mxlc.pamxctrl = &mxc;
					ZeroMemory(&mxc, sizeof(mxc));
					mxc.cbStruct = sizeof(mxc);
					rc = mixerGetLineControls((HMIXEROBJ)hMixer,&mxlc,
											   MIXER_GETLINECONTROLSF_ONEBYTYPE);
					if(rc == MMSYSERR_NOERROR)
					{
						AddVolumeControl(MAKELONG(u, 0xFFFF), mxl.szName, mxl.szName);
					}
				}
			}
			DWORD cConnections = (UINT)mxl.cConnections;
			for (DWORD v = 0; v < cConnections; v++)
			{
				MIXERLINE mxlSrc;
				mxlSrc.cbStruct      = sizeof(mxlSrc);
				mxlSrc.dwDestination = u;
				mxlSrc.dwSource      = v;
				rc = mixerGetLineInfo((HMIXEROBJ)hMixer, &mxlSrc, MIXER_GETLINEINFOF_SOURCE);
				if(MMSYSERR_NOERROR == rc)
				{
					if(mxlSrc.cControls)
					{
						MIXERLINECONTROLS mxlc;
						MIXERCONTROL mxc;

						ZeroMemory(&mxlc, sizeof(mxlc));
						mxlc.cbStruct = sizeof(mxlc);
						mxlc.dwLineID = mxlSrc.dwLineID;
						mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
						mxlc.cControls = 1;
						mxlc.cbmxctrl = sizeof(mxc);
						mxlc.pamxctrl = &mxc;
						ZeroMemory(&mxc, sizeof(mxc));
						mxc.cbStruct = sizeof(mxc);
						rc = mixerGetLineControls((HMIXEROBJ)hMixer,&mxlc,
												   MIXER_GETLINECONTROLSF_ONEBYTYPE);
						if(rc == MMSYSERR_NOERROR)
						{
							AddVolumeControl(MAKELONG(u, v), mxlSrc.szName, mxl.szName);
						}
					}
				}
			}
		}
	}
	mixerClose(hMixer);
}

void CPropsDlg::AddVolumeControl(DWORD indexes, LPCSTR name, LPCSTR dstName)
{
	HWND hLst = NULL;
	hLst = GetDlgItem(m_hWnd, IDC_LSTCONTROLS);

	int items = ListView_GetItemCount(hLst);

	LVITEM lvi;
	lvi.mask = LVIF_PARAM | LVIF_TEXT;
	lvi.iItem = items;
	lvi.iSubItem = 0;
	lvi.pszText = (LPSTR) name;
	lvi.lParam = indexes;
	ListView_InsertItem(hLst, &lvi);

	ListView_SetItemText(hLst, items, 1, (char*) dstName);

	for(int i=0; i < count; i++)
	{
		if(m_items[i] == indexes)
		{
			ListView_SetCheckState(hLst, items, TRUE);
			break;
		}
	}
}

void CPropsDlg::SaveItems()
{
	HWND hLst = GetDlgItem(m_hWnd, IDC_LSTCONTROLS);

	int itemCount = ListView_GetItemCount(hLst);
	int checkedCount = 0;
	for(int i=0; i < itemCount; i++)
	{
		if(ListView_GetCheckState(hLst, i))
		{
			checkedCount++;
		}
	}
	if(m_items)
	{
		free(m_items);
		m_items = 0;
	}
	if(checkedCount)
	{
		m_items = (DWORD*) malloc(sizeof(DWORD) * checkedCount);
	}
	
	int ii = 0;
	for(i=0; i < itemCount; i++)
	{
		if(ListView_GetCheckState(hLst, i))
		{
			LVITEM lvi;
			lvi.mask = LVIF_PARAM;
			lvi.iItem = i;
			lvi.iSubItem = 0;
			ListView_GetItem(hLst, &lvi);
			m_items[ii] = lvi.lParam;
			ii++;
		}
	}
	count = checkedCount;
}

BOOL CPropsDlg::OnEndDialog()
{
	SaveItems();

	int idx = GetCBData(IDC_SKIN);
	if(idx >=0 && idx < m_skins.GetCount())
	{
		GetRelSkinPath(m_skins[idx].m_path, m_skin);
	} else
	{
		m_skin[0] = 0;
	}

	m_isVertical = SendDlgItemMessage(m_hWnd, IDC_ISVERTICAL, BM_GETCHECK, 0, 0);
	m_showType = SendDlgItemMessage(m_hWnd, IDC_SHOWTYPE, BM_GETCHECK, 0, 0);

	if(!count) return FALSE;
	return TRUE;
}

void CPropsDlg::FillSkins()
{
	int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) "<default>");
	SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, -1);
	char path[MAX_PATH];
	GetModuleFileName(g_hInst, path, MAX_PATH);
	PathRemoveFileSpec(path);
	PathAddBackslash(path);
	strcat(path, "skins\\");

	WIN32_FIND_DATA fd;
	char findMask[MAX_PATH];
	strcpy(findMask, path);
	strcat(findMask, "*");

	HANDLE hFind = FindFirstFile(findMask, &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		do 
		{
			if(strcmp(fd.cFileName, ".") && strcmp(fd.cFileName, "..") && (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				char SkinName[255];
				char skinFile[MAX_PATH];
				strcpy(skinFile, path);
				strcat(skinFile, fd.cFileName);
				PathAddBackslash(skinFile);
				strcat(skinFile, "volskin.ini");

				GetPrivateProfileString("general", "SkinName", "", SkinName, 255, skinFile);
				if(SkinName[0])
				{
					SKIN_PATH skin;
					strcpy(skin.m_path, skinFile);
					int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) SkinName);
					SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, m_skins.GetCount());
					m_skins.Add(&skin);
				}
			}
		} 
		while(FindNextFile(hFind, &fd));
		FindClose(hFind);
	}

	SetCBData(IDC_SKIN, -1);
	char skinPath[MAX_PATH];
	GetFullSkinPath(m_skin, skinPath);

	for(int i=0; i < m_skins.GetCount(); i++)
	{
		if(!StrCmpI(m_skins[i].m_path, skinPath))
		{
			SetCBData(IDC_SKIN, i);
		}
	}
}

DWORD CPropsDlg::GetCBData(DWORD cbID)
{
	int idx = SendDlgItemMessage(m_hWnd, cbID, CB_GETCURSEL, 0, 0);
	return SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, idx, 0);
}

void CPropsDlg::SetCBData(DWORD cbID, int data)
{
	int cbCount = SendDlgItemMessage(m_hWnd, cbID, CB_GETCOUNT, 0, 0);
	for(int i=0; i < cbCount; i++)
	{
		int dt = SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, i, 0);
		if(dt == data)
		{
			SendDlgItemMessage(m_hWnd, cbID, CB_SETCURSEL, i, 0);
			break;
		}
	}
}

void CPropsDlg::SetSkin(void)
{
	char skin[MAX_PATH];
	int idx = GetCBData(IDC_SKIN);
	if(idx >=0 && idx < m_skins.GetCount())
	{
		GetRelSkinPath(m_skins[idx].m_path, skin);
	} else
	{
		skin[0] = 0;
	}
	trk->LoadSkin(skin, SendDlgItemMessage(m_hWnd, IDC_ISVERTICAL, BM_GETCHECK, 0, 0));
}

void CPropsDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_SKIN:
		if(HIWORD(itemID) == CBN_SELCHANGE)
		{
			SetSkin();
			RECT rcPreview;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);
			InvalidateRect(m_hWnd, &rcPreview, TRUE);
		}
		break;
	case IDC_ISVERTICAL:
		if(HIWORD(itemID) == BN_CLICKED)
		{
			SetSkin();
			RECT rcPreview;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);
			InvalidateRect(m_hWnd, &rcPreview, TRUE);
		}
	}
}

void CPropsDlg::OnPaint()
{
	PAINTSTRUCT ps;
	BeginPaint(m_hWnd, &ps);

	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);

	if(!SendDlgItemMessage(m_hWnd, IDC_ISVERTICAL, BM_GETCHECK, 0, 0))
	{
		int maxY = rcPreview.bottom;
		for(int i=0; i < 5; i++)
		{
			rcPreview.bottom = rcPreview.top + trk->GetHeight();
			if(rcPreview.bottom >= maxY) break;
			trk->Draw(ps.hdc, &rcPreview, rand(), RAND_MAX, 0, FALSE, i, 0);
			rcPreview.top = rcPreview.bottom + 2;
		}
	} else
	{
		int maxX = rcPreview.right;
		for(int i=0; i < 5; i++)
		{
			rcPreview.right = rcPreview.left + trk->GetHeight();
			if(rcPreview.right >= maxX) break;
			trk->Draw(ps.hdc, &rcPreview, rand(), RAND_MAX, 0, FALSE, i, 0);
			rcPreview.left = rcPreview.right + 2;
		}
	}

	EndPaint(m_hWnd, &ps);
}
