// Tracker.cpp: implementation of the CTracker class.
//
//////////////////////////////////////////////////////////////////////

#include "globals.h"
#include "Tracker.h"
#include <Shlwapi.h>
#include "resource.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTracker::CTracker()
{
	m_isVertical = FALSE;

	m_Tracker =  NULL;
	m_Ball =  NULL;
	m_Icons = NULL;
	m_Mute = NULL;

	m_TrackerAttr = NULL;
	m_BallAttr = NULL;
	m_IconsAttr = NULL;
	m_MuteAttr = NULL;

	m_TrackerLeftMargin = 0;
	m_TrackerRightMargin = 0;

	m_trackerSize.cx = 0;
	m_trackerSize.cy = 0;

	m_ballSize.cx = 0;
	m_ballSize.cy = 0;

	m_height = 0;
}

CTracker::~CTracker()
{
	if(m_Tracker)	delete m_Tracker;
	if(m_Ball)		delete m_Ball;
	if(m_Icons)		delete m_Icons;
	if(m_Mute)		delete m_Mute;

	if(m_TrackerAttr)	delete m_TrackerAttr;
	if(m_BallAttr)		delete m_BallAttr;
	if(m_IconsAttr)		delete m_IconsAttr;
	if(m_MuteAttr)		delete m_MuteAttr;
}

void CTracker::Draw(HDC dc, LPRECT rc, int value, int maxVal, int minVal, int mute, int iconIndex, int active)
{
	Graphics gr(dc);

	if(!m_isVertical)
	{
		RECT rcDraw;
		GetIconRect(rc, &rcDraw);

		gr.DrawImage(m_Icons, 
			Rect(rcDraw.left, rcDraw.top, m_iconSize.cx, m_iconSize.cy), 
			iconIndex * m_iconSize.cx, 0 , m_iconSize.cx, m_iconSize.cy, 
			UnitPixel, m_IconsAttr);

		if(mute)
		{
			gr.DrawImage(m_Mute, 
				Rect(rcDraw.left, rcDraw.top, m_muteSize.cx, m_muteSize.cy),
				0, 0 , m_muteSize.cx, m_muteSize.cy, UnitPixel, m_MuteAttr);
		}

		GetTrackerRect(rc, &rcDraw);

		if(m_TrackerLeftMargin)
		{
			gr.DrawImage(m_Tracker, 
				Rect(rcDraw.left, rcDraw.top, m_TrackerLeftMargin, m_trackerSize.cy), 
				0, active *  m_trackerSize.cy, m_TrackerLeftMargin, m_trackerSize.cy, UnitPixel, m_TrackerAttr);
		}

		gr.DrawImage(m_Tracker, 
			Rect(rcDraw.left + m_TrackerLeftMargin, rcDraw.top, rcDraw.right - rcDraw.left - m_TrackerLeftMargin - m_TrackerRightMargin, m_trackerSize.cy),
			m_TrackerLeftMargin, active *  m_trackerSize.cy, m_trackerSize.cx - m_TrackerLeftMargin - m_TrackerRightMargin, m_trackerSize.cy, 
			UnitPixel, m_TrackerAttr);

		if(m_TrackerRightMargin)
		{
			gr.DrawImage(m_Tracker, 
				Rect(rcDraw.right - m_TrackerRightMargin, rcDraw.top, m_TrackerRightMargin, m_trackerSize.cy),
				m_trackerSize.cx - m_TrackerRightMargin, active *  m_trackerSize.cy, m_TrackerRightMargin, m_trackerSize.cy, UnitPixel, m_TrackerAttr);
		}

		RECT rcSlider;
		GetSliderRect(rc, value, minVal, maxVal, &rcSlider);

		gr.DrawImage(m_Ball, 
			Rect(rcSlider.left, rcSlider.top, m_ballSize.cx, m_ballSize.cy),
			0, active *  m_ballSize.cy, m_ballSize.cx, m_ballSize.cy, UnitPixel, m_BallAttr);
	} else
	{
		RECT rcDraw;
		GetIconRect(rc, &rcDraw);

		gr.DrawImage(m_Icons, 
			Rect(rcDraw.left, rcDraw.top, m_iconSize.cx, m_iconSize.cy), 
			iconIndex * m_iconSize.cx, 0 , m_iconSize.cx, m_iconSize.cy, 
			UnitPixel, m_IconsAttr);

		if(mute)
		{
			gr.DrawImage(m_Mute, 
				Rect(rcDraw.left, rcDraw.top, m_muteSize.cx, m_muteSize.cy),
				0, 0 , m_muteSize.cx, m_muteSize.cy, UnitPixel, m_MuteAttr);
		}

		GetTrackerRect(rc, &rcDraw);

		if(m_TrackerLeftMargin)
		{
			gr.DrawImage(m_Tracker, 
				Rect(rcDraw.left, rcDraw.top, m_trackerSize.cy, m_TrackerLeftMargin), 
				active *  m_trackerSize.cy, 0, m_trackerSize.cy, m_TrackerLeftMargin, UnitPixel, m_TrackerAttr);
		}

		gr.DrawImage(m_Tracker, 
			Rect(rcDraw.left, rcDraw.top + m_TrackerLeftMargin, m_trackerSize.cy, rcDraw.bottom - rcDraw.top - m_TrackerLeftMargin - m_TrackerRightMargin),
			active *  m_trackerSize.cy, m_TrackerLeftMargin, m_trackerSize.cy, m_trackerSize.cx - m_TrackerLeftMargin - m_TrackerRightMargin, 
			UnitPixel, m_TrackerAttr);

		if(m_TrackerRightMargin)
		{
			gr.DrawImage(m_Tracker, 
				Rect(rcDraw.left, rcDraw.bottom - m_TrackerRightMargin, m_trackerSize.cy, m_TrackerRightMargin),
				active *  m_trackerSize.cy, m_trackerSize.cx - m_TrackerRightMargin, m_trackerSize.cy, m_TrackerRightMargin, UnitPixel, m_TrackerAttr);
		}

		RECT rcSlider;
		GetSliderRect(rc, value, minVal, maxVal, &rcSlider);

		gr.DrawImage(m_Ball, 
			Rect(rcSlider.left, rcSlider.top, m_ballSize.cy, m_ballSize.cx),
			active *  m_ballSize.cy, 0, m_ballSize.cy, m_ballSize.cx, UnitPixel, m_BallAttr);
	}
}

int CTracker::WherePoint(int x, int y, LPRECT rc, int value, int minVal, int maxVal, int &data)
{
	POINT pt;
	pt.x = x, pt.y = y;

	RECT rcSlider;
	GetSliderRect(rc, value, minVal, maxVal, &rcSlider);
	if(PtInRect(&rcSlider, pt))
	{
		if(!m_isVertical)
		{
			data = rcSlider.left - pt.x;
		} else
		{
			data = pt.y - rcSlider.bottom;
		}
		return 1;
	}

	RECT rcTrack;
	GetTrackerRect(rc, &rcTrack);
	if(PtInRect(&rcTrack, pt))
	{
		data = X2Value(x, minVal, maxVal, rc);
		return 2;
	}

	RECT rcMute;
	GetIconRect(rc, &rcMute);
	if(PtInRect(&rcMute, pt))
	{
		return 3;
	}

	return 0;
}

int CTracker::X2Value(int x, int minVal, int maxVal, LPRECT rc)
{
	if(!m_isVertical)
	{
		int controlWidth = TrackerWidth(rc) - m_ballSize.cx;
		int val = (x - (rc->left + m_iconSize.cx)) * (maxVal - minVal) / controlWidth;
		if(val > maxVal) val = maxVal;
		if(val < minVal) val = minVal;
		return val;
	} else
	{
		int controlWidth = TrackerWidth(rc) - m_ballSize.cx;
		int val = ((rc->bottom - m_iconSize.cy) - x) * (maxVal - minVal) / controlWidth;
		if(val > maxVal) val = maxVal;
		if(val < minVal) val = minVal;
		return val;
	}
}

void CTracker::GetTrackerRect(LPRECT rc, LPRECT rcTrk)
{
	if(!m_isVertical)
	{
		rcTrk->left = rc->left + m_iconSize.cx;
		rcTrk->right = rc->right;
		rcTrk->top = rc->top + (rc->bottom - rc->top) / 2 - m_trackerSize.cy / 2;
		rcTrk->bottom = rcTrk->top + m_trackerSize.cy;
	} else
	{
		rcTrk->left = rc->left  + (rc->right - rc->left) / 2 - m_trackerSize.cy / 2;
		rcTrk->right = rcTrk->left +  + m_trackerSize.cy;
		rcTrk->top = rc->top;
		rcTrk->bottom = rc->bottom - m_iconSize.cy;
	}
}

void CTracker::GetSliderRect(LPRECT rc, int value, int minVal, int maxVal, LPRECT rcSlider)
{
	if(!m_isVertical)
	{
		rcSlider->top = rc->top + (rc->bottom - rc->top) / 2 - m_ballSize.cy / 2;
		rcSlider->bottom = rcSlider->top + m_ballSize.cy;
		int spinX;
		int controlWidth = TrackerWidth(rc) - m_ballSize.cx;
		if(maxVal - minVal)
		{
			spinX = value * controlWidth / (maxVal - minVal);
		} else
		{
			spinX = 0;
		}
		RECT rcTracker;
		GetTrackerRect(rc, &rcTracker);
		rcSlider->left = rcTracker.left + spinX;
		rcSlider->right = rcSlider->left + m_ballSize.cx;
	} else
	{
		rcSlider->left = rc->left + (rc->right - rc->left) / 2 - m_ballSize.cy / 2;
		rcSlider->right = rcSlider->left + m_ballSize.cy;
		int spinX;
		int controlWidth = TrackerWidth(rc) - m_ballSize.cx;
		if(maxVal - minVal)
		{
			spinX = value * controlWidth / (maxVal - minVal);
		} else
		{
			spinX = 0;
		}
		RECT rcTracker;
		GetTrackerRect(rc, &rcTracker);
		rcSlider->top = rcTracker.bottom - spinX - m_ballSize.cx;
		rcSlider->bottom = rcSlider->top + m_ballSize.cx;
	}
}

void CTracker::GetIconRect(LPRECT rc, LPRECT rcIcon)
{
	if(!m_isVertical)
	{
		rcIcon->left = rc->left;
		rcIcon->right = rcIcon->left + m_iconSize.cx;
		rcIcon->top = rc->top + (rc->bottom - rc->top) / 2 - m_iconSize.cy / 2;
		rcIcon->bottom = rcIcon->top + m_iconSize.cy;
	} else
	{
		rcIcon->top = rc->bottom - m_iconSize.cy;
		rcIcon->bottom = rcIcon->top + m_iconSize.cy;
		rcIcon->left = rc->left + (rc->right - rc->left) / 2 - m_iconSize.cx / 2;
		rcIcon->right = rcIcon->left + m_iconSize.cx;
	}
}

void CTracker::LoadSkin(char* skin, BOOL isVertical)
{
	char path[MAX_PATH];

	GetFullSkinPath(skin, path);

	if(m_Tracker)	delete m_Tracker;
	if(m_Ball)		delete m_Ball;
	if(m_Icons)		delete m_Icons;
	if(m_Mute)		delete m_Mute;

	if(m_TrackerAttr)	delete m_TrackerAttr;
	if(m_BallAttr)		delete m_BallAttr;
	if(m_IconsAttr)		delete m_IconsAttr;
	if(m_MuteAttr)		delete m_MuteAttr;

	m_TrackerAttr = NULL;
	m_BallAttr = NULL;
	m_IconsAttr = NULL;
	m_MuteAttr = NULL;

	m_Tracker = NULL;
	m_Ball = NULL;
	m_Icons = NULL;
	m_Mute = NULL;

	char str[MAX_PATH];

	GetPrivateProfileString("images", "rod", "", str, MAX_PATH, path);
	if(str[0])
	{
		char imgFile[MAX_PATH];
		strcpy(imgFile, path);
		PathRemoveFileSpec(imgFile);
		PathAddBackslash(imgFile);
		strcat(imgFile, str);

		WCHAR imgFileW[MAX_PATH];

		MultiByteToWideChar(CP_ACP, 0, imgFile, -1, imgFileW, MAX_PATH);
		m_Tracker =  new Image(imgFileW);
		m_TrackerLeftMargin = GetPrivateProfileInt("rod", "leftMargin", 0, path);
		m_TrackerRightMargin = GetPrivateProfileInt("rod", "rightMargin", 0, path);
	}
	if(!m_Tracker || m_Tracker && m_Tracker->GetLastStatus() != Ok)
	{
		delete m_Tracker;
		m_Tracker = new Bitmap(g_hInst, MAKEINTRESOURCEW(IDB_PLANE));
		m_TrackerAttr = new ImageAttributes();
		m_TrackerAttr->SetColorKey(Color(0, 255, 0, 255), Color(0, 255, 0, 255), ColorAdjustTypeBitmap);
		m_TrackerLeftMargin = 6;
		m_TrackerRightMargin = 6;
	}

	GetPrivateProfileString("images", "ball", "", str, MAX_PATH, path);
	if(str[0])
	{
		char imgFile[MAX_PATH];
		strcpy(imgFile, path);
		PathRemoveFileSpec(imgFile);
		PathAddBackslash(imgFile);
		strcat(imgFile, str);

		WCHAR imgFileW[MAX_PATH];

		MultiByteToWideChar(CP_ACP, 0, imgFile, -1, imgFileW, MAX_PATH);
		m_Ball =  new Image(imgFileW);
	}
	if(!m_Ball || m_Ball && m_Ball->GetLastStatus() != Ok)
	{
		delete m_Ball;
		m_Ball = new Bitmap(g_hInst, MAKEINTRESOURCEW(IDB_BALL));
		m_BallAttr = new ImageAttributes();
		m_BallAttr->SetColorKey(Color(0, 255, 0, 255), Color(0, 255, 0, 255), ColorAdjustTypeBitmap);
	}

	GetPrivateProfileString("images", "icons", "", str, MAX_PATH, path);
	if(str[0])
	{
		char imgFile[MAX_PATH];
		strcpy(imgFile, path);
		PathRemoveFileSpec(imgFile);
		PathAddBackslash(imgFile);
		strcat(imgFile, str);

		WCHAR imgFileW[MAX_PATH];

		MultiByteToWideChar(CP_ACP, 0, imgFile, -1, imgFileW, MAX_PATH);
		m_Icons =  new Image(imgFileW);
	}
	if(!m_Icons || m_Icons && m_Icons->GetLastStatus() != Ok)
	{
		delete m_Icons;
		m_Icons = new Bitmap(g_hInst, MAKEINTRESOURCEW(IDB_ICONS));
		m_IconsAttr = new ImageAttributes();
		m_IconsAttr->SetColorKey(Color(0, 255, 0, 255), Color(0, 255, 0, 255), ColorAdjustTypeBitmap);
	}

	GetPrivateProfileString("images", "mute", "", str, MAX_PATH, path);
	if(str[0])
	{
		char imgFile[MAX_PATH];
		strcpy(imgFile, path);
		PathRemoveFileSpec(imgFile);
		PathAddBackslash(imgFile);
		strcat(imgFile, str);

		WCHAR imgFileW[MAX_PATH];

		MultiByteToWideChar(CP_ACP, 0, imgFile, -1, imgFileW, MAX_PATH);
		m_Mute =  new Image(imgFileW);
	}
	if(!m_Mute || m_Mute && m_Mute->GetLastStatus() != Ok)
	{
		delete m_Mute;
		m_Mute = new Bitmap(g_hInst, MAKEINTRESOURCEW(IDB_MUTE));
		m_MuteAttr = new ImageAttributes();
		m_MuteAttr->SetColorKey(Color(0, 255, 0, 255), Color(0, 255, 0, 255), ColorAdjustTypeBitmap);
	}

	m_iconSize.cx = m_Icons->GetWidth() / 6;
	m_iconSize.cy = m_Icons->GetHeight();

	m_muteSize.cx = m_Mute->GetWidth();
	m_muteSize.cy = m_Mute->GetHeight();

	m_trackerSize.cx = m_Tracker->GetWidth();
	m_trackerSize.cy = m_Tracker->GetHeight() / 2;

	m_ballSize.cx = m_Ball->GetWidth();
	m_ballSize.cy = m_Ball->GetHeight() / 2;

	m_height = max(m_ballSize.cy, max(m_trackerSize.cy, max(m_muteSize.cy, m_iconSize.cy)));

	m_isVertical = isVertical;
	if(m_isVertical)
	{
		m_Tracker->RotateFlip(Rotate270FlipNone);
		m_Ball->RotateFlip(Rotate270FlipNone);
	}
}
