// VolumeBtn.cpp: implementation of the CVolumeBtn class.
//
//////////////////////////////////////////////////////////////////////

#include "globals.h"
#include "VolumeBtn.h"
#include <wingdi.h>
#include "PropsDlg.h"
#include <Shlwapi.h>

void Version2Str(DWORD versionMS, DWORD versionLS, char* str)
{
	DWORD ver[4];
	ver[0] = HIWORD(versionMS);
	ver[1] = LOWORD(versionMS);
	ver[2] = HIWORD(versionLS);
	ver[3] = LOWORD(versionLS);
	int lastNum = 3;
	for(int i=3; i >=0; i--)
	{
		if(ver[i])
		{
			lastNum = i;
			break;
		}
	}
	str[0] = 0;
	for(int i=0; i <= lastNum; i++)
	{
		char num[100];
		itoa(ver[i], num, 10);
		strcat(str, num);
		if(i != lastNum)
		{
			strcat(str, ".");
		}
	}
}

void GetFileVersion(LPCTSTR fileName, DWORD& versionMS, DWORD& versionLS)
{
	versionMS = 0;
	versionLS = 0;
	DWORD dw;
	DWORD verSize = GetFileVersionInfoSize((LPTSTR) fileName, &dw);
	if(verSize)
	{
		void* verData = malloc(verSize);
		if(GetFileVersionInfo((LPTSTR) fileName, NULL, verSize, verData))
		{
			VS_FIXEDFILEINFO* verInfo;
			UINT len = 0;
			if(VerQueryValue(verData, TEXT("\\"), (void**) &verInfo, &len))
			{
				versionMS = verInfo->dwFileVersionMS;
				versionLS = verInfo->dwFileVersionLS;
			}
		}
		free(verData);
	}
}


void GetFullSkinPath(LPSTR skinPath, LPSTR fullSkinPath)
{
	GetModuleFileName(g_hInst, fullSkinPath, MAX_PATH);
	PathRemoveFileSpec(fullSkinPath);
	PathAddBackslash(fullSkinPath);
	strcat(fullSkinPath, "skins\\");
	strcat(fullSkinPath, skinPath);
}

void GetRelSkinPath(LPSTR fullSkinPath, LPSTR skinPath)
{
	char* str = StrStrI(fullSkinPath, "\\skins\\");
	if(str)
	{
		str += strlen("\\skins\\");
		strcpy(skinPath, str);
	}
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CVolumeBtn::CVolumeBtn()
{
	// Initialize GDI+.
	GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);

	m_hRes = g_hInst;

	m_showType = TRUE;
	m_isVertical = FALSE;
	m_activeControl = -1;

	m_trk = new CTracker;

	m_skin[0] = 0;

	//ptDown.x = ptDown.y = 0;
	isDown = FALSE;
	m_stop = FALSE;

	m_Name[0] = 0;
	m_state = 0;
	m_ID = 0;
	m_container = NULL;
	m_version = 4;
	m_ObjRefCount = 1;

	WNDCLASS wc;

	if(!GetClassInfo(g_hInst, "TlbVolumeControl", &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc    = (WNDPROC)CVolumeBtn::VCWndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = g_hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = NULL;
		wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = "TlbVolumeControl";

		RegisterClass(&wc);
	}

	hWndVC = NULL;

	DWORD id;
	m_WndTrd = CreateThread(NULL, 0, WindowTrdProc, this, 0, &id);

	while(!hWndVC && (DWORD)hWndVC != 0xFFFFFFFF) Sleep(10);

	m_items = NULL;
	m_count = 0;
	data = 0;

//	AddMixer(MIXERLINE_COMPONENTTYPE_DST_SPEAKERS);
//	AddMixer(MIXERLINE_COMPONENTTYPE_SRC_LINE);
//	AddMixer(MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT);
//	AddMixer(MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE);
//	AddMixer(MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC);
}

CVolumeBtn::~CVolumeBtn()
{
	if(hWndVC) 
	{
		PostMessage(hWndVC, WM_USER, 0, 0);
		WaitForSingleObject(m_WndTrd, INFINITE);
	}
	for(int i=0; i < m_count; i++)
	{
		delete m_items[i].mix;
	}
	if(m_items)
	{
		free(m_items);
	}

	delete m_trk;

	GdiplusShutdown(gdiplusToken);
}


//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CVolumeBtn::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	   {
	   *ppReturn = (IDataObject*) this;
	   }

	if(IsEqualIID(riid, IID_IUnknown))
	   {
	   *ppReturn = (IUnknown*) this;
	   }

	if(*ppReturn)
	   {
	   (*(LPUNKNOWN*)ppReturn)->AddRef();
	   return S_OK;
	   }

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CVolumeBtn::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CVolumeBtn::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}

//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////


// Load parameters for button
STDMETHODIMP CVolumeBtn::Load(IStream *data)
{
	Clear();
	DWORD loadedVersion = 0;
	ULONG cbRead = 0;
	data->Read(&loadedVersion, sizeof(DWORD), &cbRead);
	int count = 0;
	data->Read(&count, sizeof(DWORD), &cbRead);
	if(count)
	{
		DWORD* arr = (DWORD*) malloc(sizeof(DWORD) * count);
		data->Read(arr, sizeof(DWORD) * count, &cbRead);
		LoadFromArray(arr, count);
	}
	if(loadedVersion >= 2)
	{
		data->Read(m_skin, sizeof(m_skin), &cbRead);
		if(loadedVersion >= 3)
		{
			data->Read(&m_isVertical, sizeof(m_isVertical), &cbRead);
			if(loadedVersion >= 4)
			{
				data->Read(&m_showType, sizeof(m_showType), &cbRead);
			}
		}
	}

	m_trk->LoadSkin(m_skin, m_isVertical);
	RecalcLayout();
	return S_OK;
}

// Save button parameters
STDMETHODIMP CVolumeBtn::Save(IStream *data)
{
	ULONG cbWritten;
	data->Write(&m_version, sizeof(DWORD), &cbWritten);
	data->Write(&m_count, sizeof(m_count), &cbWritten);
	if(m_count)
	{
		data->Write(this->data, sizeof(DWORD) * m_count, &cbWritten);
	}
	data->Write(m_skin, sizeof(m_skin), &cbWritten);
	data->Write(&m_isVertical, sizeof(m_isVertical), &cbWritten);
	data->Write(&m_showType, sizeof(m_showType), &cbWritten);
	return S_OK;
}

// Sets button rectangle
STDMETHODIMP CVolumeBtn::SetRect(RECT *rcItem)
{
	m_rc = rcItem[0];
	return S_OK;
}

// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CVolumeBtn::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	return S_OK;
}

STDMETHODIMP CVolumeBtn::SetID(int btnID)
{
	m_ID = btnID;
	return S_OK;
}

STDMETHODIMP CVolumeBtn::Draw(HANDLE hDC, RECT *rcItem)
{
	HDC dc = (HDC) hDC;
	for(int i=0; i < m_count; i++)
	{
		RECT rcControl;
		GetControlRect(i, rcItem, &rcControl);
		m_trk->Draw(dc, &rcControl, m_items[i].mix->GetVolume(), m_items[i].mix->MaxVolume(), m_items[i].mix->MinVolume(), m_items[i].mix->isMute(), m_items[i].mix->m_iconIndex, m_activeControl == i ? 1 : 0);
	}

	return S_OK;
}

STDMETHODIMP CVolumeBtn::OnMouseOver(long keys, int x, int y)
{
	if(isDown)
	{
		RECT rcControl;
		GetControlRect(downIDX, &m_rc, &rcControl);

		int volume = 0;
		if(!m_isVertical)
		{
			volume = m_trk->X2Value(x+offcetDown, m_items[downIDX].mix->MinVolume(), m_items[downIDX].mix->MaxVolume(), &m_rc);
		} else
		{
			volume = m_trk->X2Value(y-offcetDown, m_items[downIDX].mix->MinVolume(), m_items[downIDX].mix->MaxVolume(), &m_rc);
		}
		m_items[downIDX].mix->SetVolume(volume);
	} else
	{
		int newAC = -1;
		for(int i=0; i < m_count; i++)
		{
			RECT rcControl = m_rc;
			GetControlRect(i, &m_rc, &rcControl);

			int data;
			int wherePt = m_trk->WherePoint(x, y, &rcControl, m_items[i].mix->GetVolume(), m_items[i].mix->MinVolume(), m_items[i].mix->MaxVolume(), data);
			if(wherePt)
			{
				newAC = i;
				break;
			}
		}
		if(m_activeControl != newAC)
		{
			m_activeControl = newAC;
			if(m_container)
			{
				m_container->UpdateButtonInfo(UPDATE_REDRAW);
			}
		}
	}
	return S_OK;
}

STDMETHODIMP CVolumeBtn::OnMouseLeave(long keys)
{
	isDown = FALSE;
	if(m_container && m_activeControl >= 0)
	{
		m_activeControl = -1;
		m_container->UpdateButtonInfo(UPDATE_REDRAW);
	}
	return S_OK;
}

STDMETHODIMP CVolumeBtn::OnMouseEnter(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CVolumeBtn::Create(IStream* data)
{
	if(m_container)
	{
		HWND hWndBand = m_container->GetBandWND();
		CPropsDlg dlg;
		strcpy(dlg.m_skin, m_skin);
		dlg.m_isVertical = m_isVertical;
		dlg.m_showType = m_showType;
		dlg.m_hRes = m_hRes;
		if(dlg.DoModal(hWndBand, m_hRes) == IDOK)
		{
			m_isVertical = dlg.m_isVertical;
			m_showType = dlg.m_showType;
			strcpy(m_skin, dlg.m_skin);
			LoadFromArray(dlg.m_items, dlg.count);
			return Save(data);
		}

	}
	return S_FALSE;
}


STDMETHODIMP CVolumeBtn::StateChanged(int state, int* needRedraw)
{
	needRedraw[0] = TRUE;
	m_state = state;
	return S_OK;
}

STDMETHODIMP CVolumeBtn::OnLButtonDown(long key, int x, int y)
{
	for(int i=0; i < m_count; i++)
	{
		RECT rcControl;
		GetControlRect(i, &m_rc, &rcControl);

		int data;
		int wherePt = m_trk->WherePoint(x, y, &rcControl, m_items[i].mix->GetVolume(), m_items[i].mix->MinVolume(), m_items[i].mix->MaxVolume(), data);
		if(wherePt)
		{
			if(wherePt == 1)
			{
				isDown = TRUE;
				offcetDown = data;
				downIDX = i;
				break;
			} else if(wherePt == 2)
			{
				if(!m_isVertical)
				{
					data = m_trk->X2Value(x - m_trk->GetBallWidth() / 2, m_items[i].mix->MinVolume(), m_items[i].mix->MaxVolume(), &rcControl);
				} else
				{
					data = m_trk->X2Value(y + m_trk->GetBallWidth() / 2, m_items[i].mix->MinVolume(), m_items[i].mix->MaxVolume(), &rcControl);
				}
				m_items[i].mix->SetVolume(data);
				isDown = TRUE;
				downIDX = i;
				offcetDown = - m_trk->GetBallWidth() / 2;
				break;
			}
		}
	}

	return S_FALSE;
}

STDMETHODIMP CVolumeBtn::OnLButtonUp(long key, int x, int y)
{
	isDown = FALSE;

	for(int i=0; i < m_count; i++)
	{
		RECT rcControl;
		GetControlRect(i, &m_rc, &rcControl);

		int data;
		int wherePt = m_trk->WherePoint(x, y, &rcControl, m_items[i].mix->GetVolume(), m_items[i].mix->MinVolume(), m_items[i].mix->MaxVolume(), data);
		if(wherePt)
		{
			if(wherePt == 3)
			{
				if(m_items[i].mix->isMute())
				{
					m_items[i].mix->SetMute(FALSE);
				} else
				{
					m_items[i].mix->SetMute(TRUE);
				}
				break;
			}
		}
	}

	return S_OK;
}

STDMETHODIMP CVolumeBtn::Run()
{
	//ShellExecute(NULL, NULL, "rundll32.exe", "shell32.dll,Control_RunDLL timedate.cpl", NULL, SW_NORMAL);
	return S_OK;
}


//DEL STDMETHODIMP_(int) CVolumeBtn::SupportActions()
//DEL {
//DEL 	return CTMS_PROPERTIES;
//DEL }

STDMETHODIMP CVolumeBtn::DoAction(int action)
{
	if(action == CTMS_PROPERTIES)
	{
		if(m_container)
		{
			HWND hWndBand = m_container->GetBandWND();
			CPropsDlg dlg;
			strcpy(dlg.m_skin, m_skin);
			if(m_count)
			{
				dlg.count = m_count;
				dlg.m_items = new DWORD[m_count];
				for(int i=0; i < m_count; i++)
				{
					dlg.m_items[i] = data[i];
				}
			}
			dlg.m_isVertical = m_isVertical;
			dlg.m_showType = m_showType;
			dlg.m_hRes = m_hRes;
			if(dlg.DoModal(hWndBand, m_hRes) == IDOK)
			{
				m_isVertical = dlg.m_isVertical;
				m_showType = dlg.m_showType;
				strcpy(m_skin, dlg.m_skin);
				m_trk->LoadSkin(m_skin, m_isVertical);
				LoadFromArray(dlg.m_items, dlg.count);
				m_container->UpdateButtonInfo(UPDATE_REDRAW | UPDATE_SIZE);
				return m_container->SaveButton();
			}
		}
	}
	return S_OK;
}

STDMETHODIMP_(int) CVolumeBtn::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	return 0;
}

STDMETHODIMP CVolumeBtn::InvokeCommand(int ID)
{
	return S_OK;
}


STDMETHODIMP CVolumeBtn::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	if(!hResModule)
	{
		m_hRes = g_hInst;
	} else
	{
		char flName[MAX_PATH];
		GetModuleFileName(hResModule, flName, MAX_PATH);
		VERSION flVersion;
		GetFileVersion(flName, flVersion.versionMS, flVersion.versionLS);
		VERSION minVersion;
		minVersion.versionMS = MAKELONG(1, 2);
		minVersion.versionLS = 0;
		if(flVersion >= minVersion)
		{
			m_hRes = hResModule;
		} else
		{
			m_hRes = g_hInst;
		}
	}
	return S_OK;
}

STDMETHODIMP CVolumeBtn::GetIconLocation(LPSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CVolumeBtn::SetIconLocation(LPCSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CVolumeBtn::SetName(LPCSTR szName)
{
	strcpy(m_Name, szName);
	return S_OK;
}


//DEL STDMETHODIMP_(HICON) CVolumeBtn::GetIcon()
//DEL {
//DEL 	return NULL;
//DEL }

//DEL STDMETHODIMP_(HICON) CVolumeBtn::GetSelectedIcon()
//DEL {
//DEL 	return NULL;
//DEL }

//DEL STDMETHODIMP_(LPCSTR) CVolumeBtn::GetButtonText()
//DEL {
//DEL 	return NULL;
//DEL }



//DEL void CVolumeBtn::LoadVolume()
//DEL {
//DEL 	// After successfully getting the peakmeter control, the volume range
//DEL 	// will be specified by mxc.Bounds.lMinimum to mxc.Bounds.lMaximum.
//DEL 
//DEL 	MIXERCONTROLDETAILS mxcd;             // Gets the control values.
//DEL 	MIXERCONTROLDETAILS_SIGNED volStruct; // Gets the control values.
//DEL 	//long volume;                          // Holds the final volume value.
//DEL 
//DEL 	// Initialize the MIXERCONTROLDETAILS structure
//DEL 	ZeroMemory(&mxcd, sizeof(mxcd));
//DEL 	mxcd.cbStruct = sizeof(mxcd);
//DEL 	mxcd.cbDetails = sizeof(volStruct);
//DEL 	mxcd.dwControlID = mxc.dwControlID;
//DEL 	mxcd.paDetails = &volStruct;
//DEL 	mxcd.cChannels = 1;
//DEL 
//DEL 	// Get the current value of the peakmeter control. Typically, you
//DEL 	// would set a timer in your program to query the volume every 10th
//DEL 	// of a second or so.
//DEL 	MMRESULT rc = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
//DEL 								 MIXER_GETCONTROLDETAILSF_VALUE);
//DEL 	if (MMSYSERR_NOERROR == rc) 
//DEL 	{
//DEL 		volume = volStruct.lValue;
//DEL 
//DEL 		// Get the absolute value of the volume.
//DEL 		if (volume < 0)
//DEL 			volume = -volume;
//DEL 	}
//DEL }


LRESULT CALLBACK CVolumeBtn::VCWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CVolumeBtn* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CVolumeBtn*)GetProp(hWnd, "This");
		if(pThis && pThis->hWndVC != hWnd)
		{
			pThis = NULL;
		}
	}
	if(pThis || uMessage == WM_CREATE)
	{
		//if(pThis) pThis->AddRef();
		switch (uMessage)
		{
		case WM_DESTROY:
		case WM_CLOSE:
			{
				SetProp(hWnd, "This", NULL);
			}
			break;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = (CVolumeBtn*)(lpcs->lpCreateParams);
				SetProp(hWnd, "This", (HANDLE) pThis);
				pThis->hWndVC = hWnd;
			}
			break;
		//case MM_MIXM_LINE_CHANGE:
		case MM_MIXM_CONTROL_CHANGE:
			pThis->OnChangedLine((HMIXER) wParam, (DWORD) lParam);
		}
	}
	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CVolumeBtn::OnChangedLine(HMIXER hMix, DWORD ctlID)
{
	BOOL found = FALSE;
	for(int i=0; i < m_count; i++)
	{
		if(m_items[i].mix->isThis(hMix, ctlID))
		{
			found = TRUE;
			break;
		}
	}
	if(m_container && found)
	{
		m_container->UpdateButtonInfo(UPDATE_REDRAW);
	}
}

void CVolumeBtn::AddMixer(DWORD dwComponentType)
{
	if(!m_count)
	{
		m_items = (VOLUME_CTL*) malloc(sizeof(VOLUME_CTL));
	} else
	{
		m_items = (VOLUME_CTL*) realloc(m_items, sizeof(VOLUME_CTL) * (m_count + 1));
	}
	m_items[m_count].mix = new CMixer(hWndVC, dwComponentType);
	m_count++;
	RecalcLayout();
}

void CVolumeBtn::RecalcLayout()
{
	if(!m_isVertical)
	{
		int x = 2;
		int y = 2;
		for(int i=0; i < m_count; i++)
		{
			m_items[i].pos.x = x;
			m_items[i].pos.y = y;
			y += m_trk->GetHeight() + 2;
		}
		maxHeight = y;
	} else
	{
		int x = 2;
		int y = 2;
		for(int i=0; i < m_count; i++)
		{
			m_items[i].pos.x = x;
			m_items[i].pos.y = y;
			x += m_trk->GetHeight() + 2;
		}
		maxHeight = x;
	}
}

void CVolumeBtn::Clear()
{
	if(data)
	{
		free(data);
		data = NULL;
	}
	for(int i=0; i < m_count; i++)
	{
		delete m_items[i].mix;
	}
	if(m_items)
	{
		free(m_items);
	}
	m_items = NULL;
	m_count = 0;
}

void CVolumeBtn::LoadFromArray(DWORD* data, int count)
{
	Clear();
	if(count)
	{
		for(int i=0; i < count; i++)
		{
			AddMixer(data[i]);
		}
		this->data = (DWORD*) malloc(sizeof(DWORD) * count);
		memcpy(this->data, data, sizeof(DWORD) * count);
	}
}

STDMETHODIMP CVolumeBtn::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_MODEFLAG)
	{
		btninfo->dwModeFlags = BTN_FLAG_PLANE;
		if(isDown)
		{
			btninfo->dwModeFlags |= BTN_FLAG_STOPDRAG;
		}
	}
	if(btninfo->dwMask & BTNINFO_MASK_ICON)
	{
		btninfo->hIcon = NULL;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SELECTEDICON)
	{
		btninfo->hIcon = NULL;
	}
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = CTMS_PROPERTIES;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE ||
	   btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		if(!m_isVertical)
		{
			if(btninfo->szBtnSize.cx < m_trk->GetMinWidth()) btninfo->szBtnSize.cx = m_trk->GetMinWidth();
			if(btninfo->szBtnSize.cy < maxHeight)
			{
				btninfo->szBtnSize.cy = maxHeight;
			}
		} else
		{
			btninfo->szBtnSize.cx = maxHeight;
			btninfo->szBtnSize.cy = m_trk->GetMinWidth();
		}
	}
	if(btninfo->dwMask & BTNINFO_MASK_TEXT)
	{
		btninfo->strData[0] = 0;
	}
	if(btninfo->dwMask & BTNINFO_MASK_DESCRIPTION)
	{
		btninfo->strData[0] = 0;
	}
	return S_OK;
}

STDMETHODIMP CVolumeBtn::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	switch(id)
	{
	case TBTN_INVOKE_GETTOOLTIPSCOUNT:
		{
			int* cnt = (int*) lParam;
			*cnt = m_count;
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPRECT:
		if(wParam >= 0 && (int)wParam < m_count)
		{
			LPRECT rc = (LPRECT) lParam;
			GetControlRect(wParam, &m_rc, rc);
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPTEXT:
		{
			char* str = (char*) lParam;

			if(m_showType)
			{
				strcpy(str, m_items[wParam].mix->GetTypeName());
				strcat(str, ": ");
			} else
			{
				str[0] = 0;
			}
			strcat(str, m_items[wParam].mix->GetControlName());
		}
		return S_OK;
	}
	return S_FALSE;
}


DWORD WINAPI CVolumeBtn::WindowTrdProc(LPVOID lpParam)
{
	CVolumeBtn* btn = (CVolumeBtn*) lpParam;
	//SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_IDLE);
	btn->WindowTrd();
	return 0;
}

void CVolumeBtn::WindowTrd()
{
	CreateWindow("TlbVolumeControl", "TlbVolumeControl", 0, 0, 0, 0, 0, NULL, NULL, g_hInst, this);
	if(!hWndVC)
	{
		hWndVC = (HWND) 0xFFFFFFFF;
		return;
	}

	MSG msg; 
    while (GetMessage(&msg, (HWND) hWndVC, 0, 0)) 
    { 
		if(msg.message == WM_USER)
		{
			DestroyWindow(hWndVC);
			break;
		}
        TranslateMessage(&msg); 
        DispatchMessage(&msg); 
    } 
}

void CVolumeBtn::GetControlRect(int idx, LPRECT rc, LPRECT rcControl)
{
	if(!m_isVertical)
	{
		*rcControl = *rc;
		rcControl->left = rc->left + m_items[idx].pos.x;
		rcControl->top = rc->top + m_items[idx].pos.y;
		rcControl->bottom = rcControl->top + m_trk->GetHeight();
		rcControl->right = rc->right - 2;
	} else
	{
		*rcControl = *rc;
		rcControl->left = rc->left + m_items[idx].pos.x;
		rcControl->right = rcControl->left + m_trk->GetHeight();

		rcControl->top = rc->top + m_items[idx].pos.y;
		rcControl->bottom = rc->bottom;
	}
}
